<!--<?xml version="1.0"?> -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="xml" omit-xml-declaration="yes" indent="yes" />

	<xsl:template match="/">
	<!-- PURPOSE: Main Driver for XSL transformation -->
		<xsl:variable name="xformed">
			<xsl:apply-templates select="comment()" mode="prolog" />
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$xformed = ' PCW has transformed this PCML '">
			<!-- File has been transformed already -->
				<xsl:comment>
					<xsl:value-of select="$xformed" />
				</xsl:comment>
				<xsl:copy-of select="*" />
			</xsl:when>
			<xsl:otherwise>
			<!-- File has not been transformed, add comment and begin transformation -->
				<xsl:comment>
					<xsl:value-of select="' PCW has transformed this PCML '" />
				</xsl:comment>
				<xsl:element name="pcml">
					<xsl:attribute name="version">
						<xsl:value-of select="'4.0'" />
					</xsl:attribute>
					<!-- <xsl:apply-templates select="pcml/struct"/> -->
					<!-- 
						Assumption: All structs that are not referenced do not have a use, we interact with structs 
						based on how they are referenced.  Thus, we can reach every struct by tracing down from where
						it is reference inside a program tag
					-->
					<xsl:apply-templates select="descendant::program" />
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="comment()" mode="prolog">
	<!-- PURPOSE: Check if file has been transformed already -->
		<xsl:if test=". = ' PCW has transformed this PCML '">
			<xsl:value-of select="." />
		</xsl:if>
	</xsl:template>


	<xsl:template match="program">
	<!-- PURPOSE: transform the program tag and its descendants -->
		<xsl:variable name="programname" select="@name"/>
		<xsl:for-each select="child::data">
			<xsl:variable name="structname" select="@struct" />
			<!-- 
			For each child data line that refers to a struct, find the struct in the original document and move
			it to the root of the result document.  Also append its usage from the data line to its name.
			-->
			<xsl:apply-templates select="ancestor::*/descendant::*[@name=$structname][name()='struct'][1]">
			<!--<xsl:apply-templates select="/descendant::struct[@name = $structname]"> -->
				<xsl:with-param name="usagetype" select="@usage" />
				<xsl:with-param name="programname" select="$programname" />
			</xsl:apply-templates>
		</xsl:for-each>
		<!-- for each child struct of the program,  add the struct to root of the result document -->
		<xsl:apply-templates select="child::struct">
		</xsl:apply-templates> 
		<!-- create the result program element -->
		<xsl:element name="{name()}">
			<!-- Copy only attributes name, entrypoint, epccsid, path, returnvalue, threadsafe of the program tag -->
			<!-- Skip Copy  attribute parserorder of the program tag -->
			<xsl:copy-of select="@name | @entrypoint | @epccsid | @path | @returnvalue | @threadsafe" />
			<!-- Copy everything that isn't a child struct or data tag -->
			<xsl:copy-of select="*[not(self::struct)][not(self::data)]" />
			<xsl:for-each select="child::*">
				<xsl:if test="name()='data'">
				<xsl:element name="data">
					<xsl:copy-of select="@*"/>
					<xsl:variable name="count" select="@count" />
					
					<!-- 
					Deal with each data attribute that may need its value transformed. These are: 
					
					struct - must now point to the newly named struct
					ccsid - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					length - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					name - if there is a placeholder without a name we add one to accomodate our model.
					int - Depending on its length we massage it to add default precision.
					-->
					<xsl:if test="@struct">
						<xsl:attribute name="struct">
								<xsl:for-each select="ancestor::*[@name]|.">
                                <!--<xsl:if test="not(name()='program')">--> 
                                      <xsl:choose>
                                      <xsl:when test="name() = 'data'" >
                                      
                                      <xsl:value-of select="translate(substring(@struct,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@struct) > 1" >
                                         <xsl:value-of select="substring(@struct,2,string-length(@struct) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                      </xsl:when>
                                      <xsl:when test="name() = 'program'" >
                                      
                                         <xsl:value-of select="@name"/>
                                      
                                      <xsl:value-of select="'_'"/>
                                      </xsl:when>
                                      <xsl:otherwise>
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                      </xsl:otherwise>
                                      </xsl:choose>
                                <!--  </xsl:if> --> 
                  		</xsl:for-each>
								<xsl:if test="@usage = 'output'">
									<xsl:value-of select="'Output'" />
								</xsl:if>
								<xsl:if test="@usage = 'input'">
									<xsl:value-of select="'Input'" />
								</xsl:if>
								<xsl:if test="@usage = 'inputoutput'">
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
								<xsl:if test="not(@usage)" >
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
						</xsl:attribute>
						</xsl:if>
					
					<xsl:if test="not(@usage)" >
					<xsl:attribute name="usage">
									<xsl:value-of select="'inputoutput'" />
									</xsl:attribute>
								</xsl:if>
						
					<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 
					
					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 
					
					<xsl:if test="@offsetfrom">
					<xsl:variable name="offsetfrom" select="@offsetfrom"/>
						<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
							<xsl:attribute name="offsetfrom">
  								<xsl:value-of select="$offsetfrom"/>
  								<xsl:if test="name()='struct'" >
  									<xsl:value-of select="'_Ref'"/>
  								</xsl:if>
 							</xsl:attribute>
  						</xsl:for-each>
						<xsl:if test="contains(@offsetfrom,'.')">
						<!--
						If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="offsetfrom">
							<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@offsetfrom,'.')" />
						</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
					<xsl:if test="@outputsize">
					<xsl:variable name="outputsize" select="@outputsize"/>
						<xsl:for-each select="ancestor::*/descendant::*[@name=$outputsize][1]">
							<xsl:attribute name="outputsize">
  								<xsl:value-of select="$outputsize"/>
  								<xsl:if test="name()='struct'" >
  									<xsl:value-of select="'_Ref'"/>
  								</xsl:if>
 							</xsl:attribute>
  						</xsl:for-each>
						<xsl:if test="contains(@outputsize,'.')">
						<!--
						If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="outputsize">
							<xsl:value-of select="substring-before(@outputsize,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@outputsize,'.')" />
						</xsl:attribute>
						</xsl:if>
					</xsl:if>

					<xsl:if test="not(@name)">
						<xsl:attribute name="name">
							<xsl:value-of select="'FILLER'" />
							<xsl:value-of select="count(preceding::data)" />
						</xsl:attribute>
					</xsl:if>

					<xsl:if test="@type='int'">
						<xsl:if test="not(@precision)">
							<xsl:if test="@length='2'">
								<xsl:attribute name="precision">
								<xsl:value-of select="'15'" />
								</xsl:attribute>
							</xsl:if>
							<xsl:if test="@length='4'">
								<xsl:attribute name="precision">
								<xsl:value-of select="'31'" />
								</xsl:attribute>
							</xsl:if>
							<xsl:if test="@length='8'">
								<xsl:attribute name="precision">
								<xsl:value-of select="'63'" />
								</xsl:attribute>
							</xsl:if>
						</xsl:if>
					</xsl:if>					
				
				</xsl:element>
			</xsl:if>
			
			<!-- for each child struct, replace with a data element.
			These are inlined structs.  We create a data line to refer to them, and then move the 
			struct to become a child of the root. 
			-->
			<xsl:if test="name()='struct'">
				<xsl:element name="data">
				<!--
				The new data tag can have all of the same attributes of the struct except:
				
				name: 	We are creating a new data tag without any knowledge of what data 
							tags exist already. Therefore,wemust choose a name unlikely to collide
							with an existing data element.  We use the nameof the struct and 
							concatenate it with _PCW_,and then a count that will be unique
							
				type:		The new data tag will be of type struct.
				
				struct:	The data tag will point to the struct we are moving to the root with
							its usage type appended. (and capitalized to accomodate web services)
							
				Deal with each data attribute that may need its value transformed. These are: 
					
					
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					
				-->
					<xsl:copy-of select="@*" />
					<xsl:attribute name="name">
						<xsl:value-of select="@name" />
						<!--<xsl:value-of select="'_PCW_'" />
						<xsl:value-of select="count(preceding::struct)" />-->
					</xsl:attribute>
					<xsl:attribute name="type">
						<xsl:value-of select="'struct'" />
					</xsl:attribute>
					
						<xsl:attribute name="struct">
								<xsl:for-each select="ancestor::*[@name]|.">
                                <xsl:if test="not(name()='program')">
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                  </xsl:if>
                                  <xsl:if test="name()='program'">
                                      
                                         <xsl:value-of select="@name"/>
                                      
                                      <xsl:value-of select="'_'"/>
                                  </xsl:if>
                  		</xsl:for-each>
								<xsl:if test="@usage = 'output'">
									<xsl:value-of select="'Output'" />
								</xsl:if>
								<xsl:if test="@usage = 'input'">
									<xsl:value-of select="'Input'" />
								</xsl:if>
								<xsl:if test="@usage = 'inputoutput'">
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
						</xsl:attribute>
						
<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:for-each select="ancestor::*[@name]|.">
                              	<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if>
					
					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 	
										
					<xsl:if test="contains(@outputsize,'.')">
						<!--
						If the offset value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="outputsize">
							<xsl:value-of select="substring-before(@outputsize,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@outputsize,'.')" />
						</xsl:attribute>
					</xsl:if>
					
					<xsl:if test="@offsetfrom">
					<xsl:variable name="offsetfrom" select="@offsetfrom"/>
						<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
							<xsl:attribute name="offsetfrom">
  								<xsl:value-of select="$offsetfrom"/>
  								<xsl:if test="name()='struct'" >
  									<xsl:value-of select="'_Ref'"/>
  								</xsl:if>
 							</xsl:attribute>
  						</xsl:for-each>
						<xsl:if test="contains(@offsetfrom,'.')">
						<!--
						If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="offsetfrom">
							<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@offsetfrom,'.')" />
						</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
				</xsl:element>
				</xsl:if>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<!-- match structs-->
	<xsl:template match="struct">
	<!-- PURPOSE: All structs must be referenced.  When they are referenced, this template is called.  It finds the struct
	in the document and writes it to the result document as a child of the root (pcml tag).  It also appends the usage of the 
	struct to avoid collisions if one struct is called with different usage types.
	-->

		<xsl:param name="usagetype" select="@usage" />
		<xsl:param name="programname" />
		<!-- create a new struct element -->
		<xsl:choose>
			<!--<xsl:when test="parent::pcml">-->
			 <xsl:when test="not(ancestor::program)">
				<xsl:for-each select="child::data">
				<!-- If a data element of the struct is of type struct, then go find it and write it to the result document -->
					<xsl:variable name="structname" select="@struct" />
					<xsl:apply-templates select="ancestor::*/descendant::*[@name=$structname][name()='struct'][1]">
						<xsl:with-param name="usagetype" select="$usagetype" />
						<xsl:with-param name="programname" select="$programname"/>
					</xsl:apply-templates>
				</xsl:for-each>
				
				
				<!-- Write out the struct  -->
				<xsl:element name="{name()}">
					<!-- Copy all of it's prior attributes -->
					<xsl:copy-of select="@*[generate-id(.) !=
                           generate-id(../@count)]" />
					<!--
					Deal with each attribute that may need its value transformed. These are: 
					
					name - must have called usage appended.
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					
					-->
					<!-- First write the struct's name with appended usage -->
					<xsl:attribute name="name">
						<xsl:value-of select="$programname"/>
						<xsl:value-of select="'_'"/>
						<xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:if test="not(name()='program')">
                                        <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                        <xsl:value-of select="'_'"/>
                                       </xsl:if>
                  </xsl:for-each>
						
						<xsl:choose>
						<xsl:when test="@usage = 'inherit'">
							<xsl:if test="$usagetype = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:when>
						<xsl:when test="not(@usage)">
							<xsl:if test="$usagetype = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<xsl:if test="@usage = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="@usage = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="@usage = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
					
					<xsl:if test="@usage = 'inherit'">
						<xsl:if test="$usagetype = 'output'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
								</xsl:attribute>
						</xsl:if>
						<xsl:if test="$usagetype = 'input'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
								</xsl:attribute>
							</xsl:if>
						<xsl:if test="$usagetype = 'inputoutput'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
										
					<xsl:if test="@offsetfrom">
					<xsl:variable name="offsetfrom" select="@offsetfrom"/>
						<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
							<xsl:attribute name="offsetfrom">
  								<xsl:value-of select="$offsetfrom"/>
  								<xsl:if test="name()='struct'" >
  									<xsl:value-of select="'_Ref'"/>
  								</xsl:if>
 							</xsl:attribute>
  						</xsl:for-each>
						<xsl:if test="contains(@offsetfrom,'.')">
						<!--
						If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="offsetfrom">
							<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@offsetfrom,'.')" />
						</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
					<!--
					<xsl:copy-of select="*[not(self::struct)][not(self::data)]" />
					-->
					<!-- We must maintain the order of the fields in a struct for alignment purposes. So we process each data
					or struct tag in order -->
					<xsl:for-each select="child::*">
					<xsl:if test="name() = 'data'">
						<!-- Begin data tag and copy all of its attributes -->						
						<xsl:element name="data">
						<xsl:copy-of select="@*" />
						<xsl:variable name="count" select="@count" />
						<!-- 
						Deal with each data attribute that may need its value transformed. These are: 
					
						struct - must now point to the newly named struct
						ccsid - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						length - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
						outputsize - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
						name - if there is a placeholder without a name we add one to accomodate our model.
						int - Depending on its length we massage it to add default precision.
						usage - if it is 'inherit' we need to change it.
						-->
						
						<xsl:if test="@struct">
						<xsl:attribute name="struct">
						      <xsl:value-of select="$programname"/>
						      <xsl:value-of select= "'_'"/>
								<xsl:for-each select="ancestor::*[@name]|.">
                                <xsl:if test="not(name()='program')"> 
                                <xsl:if test="not(name()='struct')"> 
                                      <xsl:choose>
                                      <xsl:when test="name() = 'data'" >
                                      
                                      <xsl:value-of select="translate(substring(@struct,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@struct) > 1" >
                                         <xsl:value-of select="substring(@struct,2,string-length(@struct) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                      </xsl:when>
                                      <xsl:otherwise>
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                      </xsl:otherwise>
                                      </xsl:choose>
                                      </xsl:if> 
                                  </xsl:if> 
                  		</xsl:for-each>
								<xsl:if test="$usagetype = 'output'">
									<xsl:value-of select="'Output'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'input'">
									<xsl:value-of select="'Input'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'inputoutput'">
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
						</xsl:attribute>
						</xsl:if>
						
<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                     	<xsl:value-of select="$programname"/>
                     	<xsl:value-of select="'.'"/>
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:for-each select="ancestor::*[@name]|.">
                              	<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if>					
						
					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                     	<xsl:value-of select="$programname"/>
                     	<xsl:value-of select="'.'"/>
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 	
						
						<xsl:if test="@offsetfrom">
						<xsl:variable name="offsetfrom" select="@offsetfrom"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
								<xsl:attribute name="offsetfrom">
  									<xsl:value-of select="$offsetfrom"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
  									</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@offsetfrom,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="offsetfrom">
								<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@offsetfrom,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
					
						<xsl:if test="@outputsize">
						<xsl:variable name="outputsize" select="@outputsize"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$outputsize][1]">
								<xsl:attribute name="outputsize">
  									<xsl:value-of select="$outputsize"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
 	 								</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@outputsize,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="outputsize">
								<xsl:value-of select="substring-before(@outputsize,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@outputsize,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
						
						<xsl:if test="@usage = 'inherit'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
						<xsl:if test="not(@usage)">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
						
						<xsl:if test="not(@name)">
									<xsl:attribute name="name">
										<xsl:value-of select="'FILLER'" />
										<xsl:value-of select="count(preceding::data)" />
									</xsl:attribute>
						</xsl:if>
						
						<xsl:if test="@type='int'">
									<xsl:if test="not(@precision)">
										<xsl:if test="@length='2'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'15'" />
											</xsl:attribute>
										</xsl:if>
										<xsl:if test="@length='4'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'31'" />
											</xsl:attribute>
										</xsl:if>
										<xsl:if test="@length='8'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'63'" />
											</xsl:attribute>
										</xsl:if>
									</xsl:if>
						</xsl:if>
						
						</xsl:element>
					</xsl:if>

					
					<!-- for each child struct, replace with a data element.
					These are inlined structs.  We create a data line to refer to them, and then move the 
					struct to become a child of the root. 
					-->
					<xsl:if test="name() = 'struct'">
					<!--
					The new data tag can have all of the same attributes of the struct except:
				
					name: 	We are creating a new data tag without any knowledge of what data 
								tags exist already. Therefore,wemust choose a name unlikely to collide
								with an existing data element.  We use the nameof the struct and 
								concatenate it with _PCW_,and then a count that will be unique
							
					type:		The new data tag will be of type struct.
				
					struct:	The data tag will point to the struct we are moving to the root with
								its usage type appended. (and capitalized to accomodate web services)
							
					Deal with each data attribute that may need its value transformed. These are: 
						
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					usage - the usage of the struct.
					-->
						<xsl:element name="data">
							<xsl:copy-of select="@*" />
							<xsl:attribute name="name">
								<xsl:value-of select="@name" />
								<!--<xsl:value-of select="'_PCW_'" />
						<xsl:value-of select="count(preceding::struct)" />-->
							</xsl:attribute>
							
							<xsl:attribute name="type">
								<xsl:value-of select="'struct'" />
							</xsl:attribute>
							
							<xsl:attribute name="struct">
							<xsl:value-of select="$programname"/>
							<xsl:value-of select="'_'"/>
								<xsl:for-each select="ancestor::*[@name]|.">
                                <xsl:if test="not(name()='program')">
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                  </xsl:if>
                                  
                  		</xsl:for-each>
								<xsl:if test="$usagetype = 'output'">
									<xsl:value-of select="'Output'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'input'">
									<xsl:value-of select="'Input'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'inputoutput'">
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
							</xsl:attribute>
							
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
							
<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                     	<xsl:value-of select="$programname"/>
                     	<xsl:value-of select="'.'"/>
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:for-each select="ancestor::*[@name]|.">
                              	<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if>
                  
 					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                     	<xsl:value-of select="$programname"/>
                     	<xsl:value-of select="'.'"/>
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 
                  
						<xsl:if test="contains(@outputsize,'.')">
						<!--
						If the offset value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
							<xsl:attribute name="outputsize">
								<xsl:value-of select="substring-before(@outputsize,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@outputsize,'.')" />
							</xsl:attribute>
						</xsl:if>
					
						<xsl:if test="@offsetfrom">
						<xsl:variable name="offsetfrom" select="@offsetfrom"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
								<xsl:attribute name="offsetfrom">
  									<xsl:value-of select="$offsetfrom"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
  									</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@offsetfrom,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="offsetfrom">
								<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@offsetfrom,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
											
						</xsl:element>
					</xsl:if>
					</xsl:for-each>
				</xsl:element>
				
			</xsl:when>
			<xsl:otherwise>
			
			<!-- This struct is not a child of the root -->
				<xsl:element name="{name()}">
					<!-- Copy all of it's prior attributes -->
					<xsl:copy-of select="@*[generate-id(.) !=
                           generate-id(../@count)]" />
					<!--
					Deal with each attribute that may need its value transformed. These are: 
					
					name - must have called usage appended.
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					
					-->
					<!-- First write the struct's name with appended usage -->
					<xsl:attribute name="name">
						
						<xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:if test="not(name()='program')">
                                        <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                        <xsl:value-of select="'_'"/>
                                 </xsl:if> 
                                 <xsl:if test="name()='program'">
                                        <xsl:value-of select="@name"/>
                                        <xsl:value-of select="'_'"/>
                                 </xsl:if>
                  </xsl:for-each>
						
						<xsl:choose>
						<xsl:when test="@usage = 'inherit'">
							<xsl:if test="$usagetype = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:when>
						<xsl:when test="not(@usage)">
							<xsl:if test="$usagetype = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<xsl:if test="@usage = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="@usage = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="@usage = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
						</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
					
					<xsl:if test="@usage = 'inherit'">
						<xsl:if test="$usagetype = 'output'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
								</xsl:attribute>
						</xsl:if>
						<xsl:if test="$usagetype = 'input'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
								</xsl:attribute>
							</xsl:if>
						<xsl:if test="$usagetype = 'inputoutput'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
					
					
					<xsl:if test="contains(@outputsize,'.')">
						<!--
						If the offset value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="outputsize">
							<xsl:value-of select="substring-before(@outputsize,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@outputsize,'.')" />
						</xsl:attribute>
					</xsl:if>
					
					<xsl:if test="@offsetfrom">
					<xsl:variable name="offsetfrom" select="@offsetfrom"/>
						<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
							<xsl:attribute name="offsetfrom">
  								<xsl:value-of select="$offsetfrom"/>
  								<xsl:if test="name()='struct'" >
  									<xsl:value-of select="'_Ref'"/>
  								</xsl:if>
 							</xsl:attribute>
  						</xsl:for-each>
						<xsl:if test="contains(@offsetfrom,'.')">
						<!--
						If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
						<xsl:attribute name="offsetfrom">
							<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
							<xsl:value-of select="'_Ref'" />
							<xsl:value-of select="'.'" />
							<xsl:value-of select="substring-after(@offsetfrom,'.')" />
						</xsl:attribute>
						</xsl:if>
					</xsl:if>
					
					
										
					<!--  <xsl:copy-of select="@*[not(name()='usage')]"/> -->
					<!-- for each data element replace 'inherit' attribute with provided usage -->
					<xsl:for-each select="child::*">
					<!-- Maintain order of structs children -->
					<!-- Begin data tag and copy all of its attributes -->						
					<xsl:if test="name()='data'">
						<xsl:element name="data">
						<xsl:copy-of select="@*" />
						<xsl:variable name="count" select="@count" />
						<!-- 
						Deal with each data attribute that may need its value transformed. These are: 
					
						struct - must now point to the newly named struct
						ccsid - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						length - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
						offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
						outputsize - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
						name - if there is a placeholder without a name we add one to accomodate our model.
						int - Depending on its length we massage it to add default precision.
						usage - if it is 'inherit' we need to change it.
						-->
						
						
						<xsl:if test="@struct">
							<xsl:attribute name="struct">
								<xsl:value-of select="$programname"/>
								<xsl:value-of select="'_'" />
								<xsl:value-of select="@struct" />
								<xsl:value-of select="'_'" />
							<xsl:if test="$usagetype = 'output'">
								<xsl:value-of select="'Output'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'input'">
								<xsl:value-of select="'Input'" />
							</xsl:if>
							<xsl:if test="$usagetype = 'inputoutput'">
								<xsl:value-of select="'InputOutput'" />
							</xsl:if>
							</xsl:attribute>
						</xsl:if>
						
<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:for-each select="ancestor::*[@name]|.">
                              	<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if>
						
					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 
						
						<xsl:if test="@offsetfrom">
						<xsl:variable name="offsetfrom" select="@offsetfrom"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
								<xsl:attribute name="offsetfrom">
  									<xsl:value-of select="$offsetfrom"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
  									</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@offsetfrom,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="offsetfrom">
								<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@offsetfrom,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
					
						<xsl:if test="@outputsize">
						<xsl:variable name="outputsize" select="@outputsize"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$outputsize][1]">
								<xsl:attribute name="outputsize">
  									<xsl:value-of select="$outputsize"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
 	 								</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@outputsize,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="outputsize">
								<xsl:value-of select="substring-before(@outputsize,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@outputsize,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
						
						<xsl:if test="@usage = 'inherit'">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
						<xsl:if test="not(@usage)">
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
						</xsl:if>
						
						<xsl:if test="not(@name)">
									<xsl:attribute name="name">
										<xsl:value-of select="'FILLER'" />
										<xsl:value-of select="count(preceding::data)" />
									</xsl:attribute>
						</xsl:if>
						
						<xsl:if test="@type='int'">
									<xsl:if test="not(@precision)">
										<xsl:if test="@length='2'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'15'" />
											</xsl:attribute>
										</xsl:if>
										<xsl:if test="@length='4'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'31'" />
											</xsl:attribute>
										</xsl:if>
										<xsl:if test="@length='8'">
											<xsl:attribute name="precision">
												<xsl:value-of select="'63'" />
											</xsl:attribute>
										</xsl:if>
									</xsl:if>
						</xsl:if>
						
						</xsl:element>
					</xsl:if>
					<!-- for each child struct, replace with a data element -->
					<!-- for each child struct, replace with a data element.
					These are inlined structs.  We create a data line to refer to them, and then move the 
					struct to become a child of the root. 
					-->
					<xsl:if test="name() = 'struct'">
					<!--
					The new data tag can have all of the same attributes of the struct except:
				
					name: 	We are creating a new data tag without any knowledge of what data 
								tags exist already. Therefore,wemust choose a name unlikely to collide
								with an existing data element.  We use the nameof the struct and 
								concatenate it with _PCW_,and then a count that will be unique
							
					type:		The new data tag will be of type struct.
				
					struct:	The data tag will point to the struct we are moving to the root with
								its usage type appended. (and capitalized to accomodate web services)
							
					Deal with each data attribute that may need its value transformed. These are: 
						
					count - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offset - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					outputsize - if the value is non numeric then it points to a data tag.  If it points to
							a struct.data-name then we point to newly named struct.
					offsetfrom - if the value is non numeric then it points to a data or struct tag.  
							If it points to a struct.data-name or a struct, then we point to newly named struct.
					usage - the usage of the struct.
					-->
						<xsl:element name="data">
							<xsl:copy-of select="@*" />
							<xsl:attribute name="name">
								<xsl:value-of select="@name" />
								<!--<xsl:value-of select="'_PCW_'" />
						<xsl:value-of select="count(preceding::struct)" />-->
							</xsl:attribute>
							
							<xsl:attribute name="type">
								<xsl:value-of select="'struct'" />
							</xsl:attribute>
							
							<xsl:attribute name="struct">
								<xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:if test="not(name()='program')">
                                        <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                        <xsl:value-of select="'_'"/>
                                       </xsl:if>
                                 <xsl:if test="name()='program'">
                                        <xsl:value-of select="@name"/>
                                        <xsl:value-of select="'_'"/>
                                       </xsl:if>
                  		</xsl:for-each>
								<xsl:if test="$usagetype = 'output'">
									<xsl:value-of select="'Output'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'input'">
									<xsl:value-of select="'Input'" />
								</xsl:if>
								<xsl:if test="$usagetype = 'inputoutput'">
									<xsl:value-of select="'InputOutput'" />
								</xsl:if>
							</xsl:attribute>
							
							<xsl:attribute name="usage">
								<xsl:value-of select="$usagetype" />
							</xsl:attribute>
							
<xsl:if test="@count">	
						 <xsl:choose>
						 <xsl:when test="number(@count)=@count">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@count,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theCount" select="@count"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theCount][1]">
                     	<xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:for-each select="ancestor::*[@name]|.">
                              	<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if>
						
					<xsl:if test="@length">	
						 <xsl:choose>
						 <xsl:when test="number(@length)=@length">
						 </xsl:when>
						 <xsl:otherwise>
						  <xsl:choose>
						  <xsl:when test="contains(@length,'.')">
						 <!-- <xsl:variable name="theCountparent" select="substring-before(@count,'.')" />
						  <xsl:attribute name="count">
                        <xsl:for-each select="ancestor::*/descendant::*[@name=$theCountparent][1]">
                    			<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                                  
                                      <xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                      <xsl:if test ="string-length(@name) > 1" >
                                         <xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                      </xsl:if>
                                      <xsl:value-of select="'_'"/>
                                 
                  		      </xsl:for-each>
                              <xsl:if test="@usage = 'output'">
											<xsl:value-of select="'Output'" />
										</xsl:if>
										<xsl:if test="@usage = 'input'">
											<xsl:value-of select="'Input'" />
										</xsl:if>
										<xsl:if test="@usage = 'inputoutput'">
											<xsl:value-of select="'InputOutput'" />
										</xsl:if>
                         </xsl:if>
                         <xsl:if test="name()='program'">
                             <xsl:value-of select="@name"/>
                         </xsl:if>
                      </xsl:for-each>    
                      <xsl:value-of select="'.'"/>
                      <xsl:value-of select="substring-after(@count,'.')"/>
                    </xsl:attribute> -->
						  </xsl:when>
						  <xsl:otherwise> 
					      <xsl:variable name="theLength" select="@length"/>
                     <xsl:for-each select="ancestor::*/descendant::*[@name=$theLength][1]">
                     	<xsl:attribute name="length">
                        <xsl:for-each select="ancestor::*[1]">
                        	<xsl:if test="name()='struct'" >
                           	<xsl:for-each select="ancestor::*[@name]|.">
                              	<!--<xsl:value-of select="translate(substring(@name,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                                 <xsl:if test ="string-length(@name) > 1" >
                                 	<xsl:value-of select="substring(@name,2,string-length(@name) - 1)"/>
                                 </xsl:if>-->
                                 <xsl:value-of select="@name"/>
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                           <xsl:if test="name()='program'" >
                              <xsl:for-each select="ancestor::*[@name]|.">
                                 <xsl:value-of select="@name" /> 
                                 <xsl:value-of select="'.'"/>
                  		      </xsl:for-each>
                           </xsl:if>
                        </xsl:for-each>
                        <xsl:value-of select="@name"/>
                        </xsl:attribute>
                     </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                  </xsl:choose>
                  </xsl:if> 
												
						<xsl:if test="contains(@outputsize,'.')">
						<!--
						If the offset value contains a '.' then we know it is non numeric and referencing a data member
						of a struct. We must replace the left side (struct name) with appropriate value.
						-->
							<xsl:attribute name="outputsize">
								<xsl:value-of select="substring-before(@outputsize,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@outputsize,'.')" />
							</xsl:attribute>
						</xsl:if>
					
						<xsl:if test="@offsetfrom">
						<xsl:variable name="offsetfrom" select="@offsetfrom"/>
							<xsl:for-each select="ancestor::*/descendant::*[@name=$offsetfrom][1]">
								<xsl:attribute name="offsetfrom">
  									<xsl:value-of select="$offsetfrom"/>
  									<xsl:if test="name()='struct'" >
  										<xsl:value-of select="'_Ref'"/>
  									</xsl:if>
 								</xsl:attribute>
  							</xsl:for-each>
							<xsl:if test="contains(@offsetfrom,'.')">
							<!--
							If the offsetfrom value contains a '.' then we know it is non numeric and referencing a data member
							of a struct. We must replace the left side (struct name) with appropriate value.
							-->
							<xsl:attribute name="offsetfrom">
								<xsl:value-of select="substring-before(@offsetfrom,'.')" />						
								<xsl:value-of select="'_Ref'" />
								<xsl:value-of select="'.'" />
								<xsl:value-of select="substring-after(@offsetfrom,'.')" />
							</xsl:attribute>
							</xsl:if>
						</xsl:if>
											
						</xsl:element>
					</xsl:if>
					</xsl:for-each>
				</xsl:element>
				
			</xsl:otherwise>
			
		</xsl:choose>
		<!-- for each child struct, add to result document -->
		<xsl:apply-templates select="child::struct">
			<xsl:with-param name="usagetype" select="$usagetype" />
			<xsl:with-param name="programname" select="$programname" />
		</xsl:apply-templates>
	</xsl:template>
</xsl:stylesheet>



